#include <avr/io.h>
#include <avr/interrupt.h>
#include "Servo.h"
#include "WProgram.h"
#include "../uart.h"

servo_t servos[SERVO_COUNT];
volatile uint8_t usedServos = 0;
volatile uint8_t loopIndex = 0;

void initServos() {
	// Setup timer
	TCCR1A = 0;
	OCR1B = 0xFFFF; // turn off point
	OCR1A = SERVO_CYCLE; // next cycle point
	TCCR1B = _BV(WGM12) | _BV(CS11); // CTC @ OCR1A, clk/8 => 1us/tick
	TIMSK |= _BV(OCIE1A) | _BV(OCIE1B); // enable compare interrupts
	sei();
}

ISR(TIMER1_COMPB_vect) { // Turn off the servo (midpoint)
	servo_t servo = servos[loopIndex];
	if (servo.pin > 0) {
		*servo.port &= ~_BV(servo.pin);
	}		
}

ISR(TIMER1_COMPA_vect) { // Switch to next servo
	loopIndex = (loopIndex+1) % SERVO_COUNT;
	servo_t servo = servos[loopIndex];
	TCNT1 = 0;

	if (servo.onTime > 0) {
		OCR1B = servo.onTime;
		*servo.port |= _BV(servo.pin);
	} else OCR1B = 0xFFFF;
}

Servo::Servo() {}

void Servo::init(volatile uint8_t pin, volatile uint8_t* port, volatile uint8_t* ddr) {
	this->init(pin, port, ddr, MIN_PULSE_WIDTH, MAX_PULSE_WIDTH);
}

void Servo::init(uint8_t pin, volatile uint8_t* port, volatile uint8_t* ddr, uint16_t min, uint16_t max) {
	this->servoIndex = usedServos++;
	this->min = min;
	this->max = max;
	*ddr |= _BV(pin); // make output	
	
	// Setup the servo entry
	servo_t servo;
	servo.pin = pin;	
	servo.port = port;
	servo.ddr = ddr;
	servo.onTime = 0;
	servos[this->servoIndex] = servo;
}

void Servo::write(uint8_t value) {
	uint32_t diff = (this->max - this->min);
	uint16_t pulse = (uint32_t)(value * diff / 255UL + this->min);
	//uart_putstr(" T="); uart_putc(((this->max / 1000) % 10) + '0'); uart_putc(((this->max / 100) % 10) + '0');
	//uart_putstr(" B="); uart_putc(((this->min / 1000) % 10) + '0'); uart_putc(((this->min / 100) % 10) + '0');
	//uart_putstr(" P="); uart_putc(((pulse / 1000) % 10) + '0'); uart_putc(((pulse / 100) % 10) + '0');
	this->writeMicroseconds(pulse);
}	

void Servo::writeMicroseconds(uint16_t value) {
	servo_t servo = servos[this->servoIndex];
	servo.onTime = value;
	servos[this->servoIndex] = servo;
}