#ifndef Servo_h
#define Servo_h

#define MIN_PULSE_WIDTH       600UL     // the shortest pulse sent to a servo  
#define MAX_PULSE_WIDTH      2300UL     // the longest pulse sent to a servo 
#define DEFAULT_PULSE_WIDTH  1500UL     // default pulse width when servo is attached

#define SERVO_CYCLE          6666UL
#define REFRESH_INTERVAL    20000UL     // minimum time to refresh servos in microseconds 
#define SERVO_COUNT	        3     // 8 * 2500 = 20ms

typedef struct {
  uint8_t pin;
  volatile uint8_t* port;
  volatile uint8_t* ddr;
  uint16_t onTime;
} servo_t;

void initServos();

class Servo
{
public:
  Servo();
  void init(uint8_t pin, volatile uint8_t* port, volatile uint8_t* ddr);
  void init(uint8_t pin, volatile uint8_t* port, volatile uint8_t* ddr, uint16_t min, uint16_t max);
  void write(uint8_t value);             // 0 = 0%, 255 = 100%
  void writeMicroseconds(uint16_t value); // Write pulse width in microseconds 
private:
   uint8_t servoIndex;               // index into the channel data for this servo
   uint16_t min;                       // minimum is this value times 4 added to MIN_PULSE_WIDTH    
   uint16_t max;                       // maximum is this value times 4 added to MAX_PULSE_WIDTH   
};

#endif